# Copyright (C) 2022-2023 |Meso|Star> (contact@meso-star.com)
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

cmake_minimum_required(VERSION 3.1)
include(SelectLibraryConfigurations)

# Try to find the libcyaml devel. Once done this will define:
#   - libcyaml_FOUND: system has libcyaml
#   - libcyaml_INCLUDE_DIR: the include directory
#   - libcyaml: Link this to use libcyaml

# Look for library header
find_path(libcyaml_INCLUDE_DIR cyaml/cyaml.h)

# Look for Release, Debug, RelWithDebInfo and MinSizeRel libraries
unset(libcyaml_LIBRARY CACHE)
unset(libcyaml_LIBRARY_RELEASE CACHE)
unset(libcyaml_LIBRARY_DEBUG CACHE)
unset(libcyaml_LIBRARY_RELWITHDEBINFO CACHE)
unset(libcyaml_LIBRARY_MINSIZEREL CACHE)

# Find per configuration type libraries
find_library(libcyaml_LIBRARY_RELEASE
  cyaml
  PATH_SUFFIXES bin Bin BIN
  DOC "Path to the library libcyaml used during release builds.")
find_library(libcyaml_LIBRARY_DEBUG
  cyaml-dbg
  PATH_SUFFIXES bin Bin BIN
  DOC "Path to the library libcyaml used during debug builds.")
find_library(libcyaml_LIBRARY_RELWITHDEBINFO
  cyaml-rdbg
  PATH_SUFFIXES bin Bin BIN
  DOC "Path to the library libcyaml used during release with debug info builds.")
find_library(libcyaml_LIBRARY_MINSIZEREL
  cyaml-mszr
  PATH_SUFFIXES bin Bin BIN
  DOC "Path to the library libcyaml used during minsize builds.")

# Define the generic libcyaml library
if(libcyaml_LIBRARY_RELEASE)
  set(libcyaml_LIBRARY ${libcyaml_LIBRARY_RELEASE})
elseif(libcyaml_LIBRARY_RELWITHDEBINFO)
  set(libcyaml_LIBRARY ${libcyaml_LIBRARY_RELWITHDEBINFO})
elseif(libcyaml_LIBRARY_MINSIZEREL)
  set(libcyaml_LIBRARY ${libcyaml_LIBRARY_MINSIZEREL})
elseif(libcyaml_LIBRARY_DEBUG)
  set(libcyaml_LIBRARY ${libcyaml_LIBRARY_DEBUG})
endif()

# Define the per configuration library fallback when not found
set(_configs RELEASE DEBUG RELWITHDEBINFO MINSIZEREL)
foreach(_cfg ${_configs})
  if(NOT libcyaml_LIBRARY_${_cfg})
    get_property(_doc CACHE libcyaml_LIBRARY_${_cfg} PROPERTY HELPSTRING)
    set(libcyaml_LIBRARY_${_cfg}
      ${libcyaml_LIBRARY} CACHE PATH ${_doc} FORCE)
  endif()
endforeach()

# Create the imported library target
add_library(libcyaml SHARED IMPORTED)

# Setup the properties of the imported target
if(CMAKE_HOST_WIN32)
  set(_import_prop IMPORTED_IMPLIB)
else()
  set(_import_prop IMPORTED_LOCATION)
endif()
set_target_properties(libcyaml PROPERTIES
  ${_import_prop} ${libcyaml_LIBRARY}
  ${_import_prop}_RELEASE ${libcyaml_LIBRARY_RELEASE}
  ${_import_prop}_DEBUG ${libcyaml_LIBRARY_DEBUG}
  ${_import_prop}_RELWITHDEBINFO ${libcyaml_LIBRARY_RELWITHDEBINFO}
  ${_import_prop}_MINSIZEREL ${libcyaml_LIBRARY_MINSIZEREL}
  INTERFACE_INCLUDE_DIRECTORIES ${libcyaml_INCLUDE_DIR})

# Check the package
include(FindPackageHandleStandardArgs)
FIND_PACKAGE_HANDLE_STANDARD_ARGS(libcyaml DEFAULT_MSG
  libcyaml_INCLUDE_DIR
  libcyaml_LIBRARY)
